<?php
  include_once('plugin_class_root.inc.php');
  include_once(PHPWG_PLUGINS_PATH.'grum_plugins_classes/tabsheets.class.inc.php');
  include_once(PHPWG_PLUGINS_PATH.'grum_plugins_classes/translate.class.inc.php');
  include_once(PHPWG_PLUGINS_PATH.'grum_plugins_classes/css.class.inc.php');
  include_once(PHPWG_PLUGINS_PATH.'grum_plugins_classes/pages_navigation.class.inc.php');

  /* classe translator pour l'administration */
  class bo_translator extends root_translator
  {
    var $tabsheets;
    var $css;   //the css object

    function bo_translator($prefixeTable, $filelocation)
    {
      parent::root_translator($prefixeTable, $filelocation);
      $this->css = new css(dirname($this->filelocation).'/'.$this->plugin_name.".css");
    }

    /*
      surchage of common_plugin->save_config function
    */
    function save_config()
    {
      if(parent::save_config())
      {
        $this->css->make_CSS($this->generate_CSS());
        return(true);
      }
      return(false);
    }

    /*
      surchage of common_plugin->load_config function
    */
    function load_config()
    {
      parent::load_config();
      if(!$this->css->css_file_exists())
      {
        $this->css->make_CSS($this->generate_CSS());
      }
    }


    /*
      initialize events call for the plugin
    */
    function init_events()
    {
      add_event_handler('loc_end_page_header', array(&$this->css, 'apply_CSS'));
      add_event_handler('get_admin_plugin_menu_links', array(&$this, 'plugin_admin_menu'));
      //$this->css->make_CSS($this->generate_CSS());
    }

    /*
      generate the css code
    */
    function generate_CSS()
    {
      $text = "
.g001_graphbar1,
.g001_graphbar2,
.g001_graphbar3,
.g001_graphbar4,
.g001_graphbarx {
    border:0px;
    height:8px;
    display: block;
    float:left;
    margin:0px;
    padding:0px;
    position:relative;
    top:3px;
  }
.g001_graphbar1 { background-color:#".$this->my_config['color_bars1']."; }
.g001_graphbar2 { background-color:#".$this->my_config['color_bars2']."; }
.g001_graphbar3 { background-color:#".$this->my_config['color_bars3']."; }
.g001_graphbar4 { background-color:#".$this->my_config['color_bars4']."; }
.g001_graphbarx { background-color:transparent; top:0px; height:1px; }

.g001_table_row:hover
{
  background-color:#".$this->my_config['mouse_over_color'].";
  cursor:pointer;
}

.g001_table_row
{
  vertical-align:top;
}


.g001_table_r
{
  margin:2px; //2px
  padding:2px;
  text-align:left;
  font-size:90%;
  border-style:hidden;
  border-spacing:0px 1px;
}

.g001_table_r TH
{
  text-align:center;
  border-spacing:2px;
  padding:2px;
}

.g001_table_r TD
{
  padding-left:3px;
  padding-right:3px;
}


.g001_table_nfo
{
  text-align:left;
}

.g001_menu
{
  padding:5px;
  background:#".$this->my_config['mouse_over_color'].";
  z-index:10000;
  font-size:90%;
}

.ttnfo
{
  color:#".$this->my_config['color_bars1'].";
}
.tvnfo
{
  color:#".$this->my_config['color_bars2'].";
}
.tonfo
{
  color:#".$this->my_config['color_bars3'].";
}
.tdnfo
{
  color:#".$this->my_config['color_bars4'].";
}
      ";
      return($text);
    }



    /* 
      display administration page
    */
    function display_admin_page()
    {
      global $template, $user;


      //tabsheet management
      if(!isset($_REQUEST['f_tabsheet']))
      {
        $_REQUEST['f_tabsheet']='status';
      }

      $this->tabsheets=new tabsheets();
      $this->tabsheets->set_titlename('TABSHEET_TITLE');
      $this->tabsheets->add('status',
                            'g001_status',
                            $this->page_link."&f_tabsheet=status", true);
      if($_REQUEST['f_tabsheet']=='editing')
      {
        $this->tabsheets->add('editing', 'g001_editing','');
      }
      $this->tabsheets->add('langs', 'g001_langs',
                            $this->page_link."&f_tabsheet=langs");


      $this->tabsheets->add('maintenance', 'g001_maintenance',
                            $this->page_link."&f_tabsheet=maintenance");


      $this->tabsheets->add('config', 'g001_config',
                            $this->page_link."&f_tabsheet=config");


      $this->load_config();
      $this->tabsheets->select($_REQUEST['f_tabsheet']);
      $this->tabsheets->make_tabsheets();

      switch($_REQUEST['f_tabsheet'])
      {
        case 'status':
          $this->display_status_page();
          /*
            status page displays list of all .lang.php file (by row) and available languages (by col)
              for each file possibility to :
                - compare translation => links to EDITING page
              for each file/language possibility to :
                - translate into an another available language => links to EDITING page
                - edit the file/language => links to EDITING page
          */
          break;
        case 'langs':
          $this->display_langs_page();
          /*
            langs page display list of all available language
            possibility to :
              - add a language => links to LANGS page
            for each language, possibility to :
              - edit language => links to LANGS page
              - delete language (sub edit function) => links to LANGS page
          */
          break;
        case 'maintenance':
          $this->display_maintenance_page();
          /*
            maintenance page allows to :
              - export database translation to .lang.php files (implementation not finished)  => links to MAINTENANCE/EXPORT page
              - purge database => links to MAINTENANCE/EXPORT page
                  consist to purge deleted items or to clean all items versions (only keep the last version of an item)
              - analyse differences between database translated files and disk's files (not implemented yet)
          */
          break;
        case 'editing':
          $this->display_editing_page();
          /*
            editing page allows to :
              - edit items of language file
              - compare languages file (display keys difference between languages files) 
          */
          break;
        case 'config': 
          $this->display_config_page();
          /*
            config page allows to :
              - set plugin's options
          */
          break;
      }

      //translator mode management
      if(isset($user['translator_mode']))
      {
        $translator_mode=l10n('g001_translator_mode_is_on');
        $translator_alternate=l10n('g001_deactivate_translator_mode');
        $modeonoff='off';
      }
      else
      {
        $translator_mode=l10n('g001_translator_mode_is_off');
        $translator_alternate=l10n('g001_activate_translator_mode');
        $modeonoff='on';
      }

      $template->assign_vars(
        array(
          'DISP_TRANSLATOR_MODE' => $translator_mode,
          'DISP_TRANSLATOR_ALTERNATE' => $translator_alternate,
          'AJAX_URL_TRANSLATOR_MODE' => $this->page_link."&ajaxfct=switch_tmode&f_translator_mode=".$modeonoff
        )
      );
    }

    /*
      return ajax content
    */
    function return_ajax_content()
    { 
      if(isset($_REQUEST['ajaxfct']))
      {
        $result="<p class='errors'>An error has occured</p>";
        switch($_REQUEST['ajaxfct'])
        {
          case 'translation_detail_list':
            $result=$this->ajax_translation_detail_list();
            break;
          case 'do_translation':
            $result=$this->ajax_do_translation();
            break;
          case 'file_nfo':
            $result=$this->ajax_file_nfo();
            break;
          case 'load_edit_list':
            $result=$this->ajax_load_edit_list();
            break;
          case 'key_nfo':
            $result=$this->ajax_key_nfo();
            break;
          case 'key_value':
            $result=$this->ajax_key_value();
            break;
          case 'validate_translation':
            $result=$this->ajax_validate_translation();
            break;
          case 'validate_all':
            $result=$this->ajax_validate_all();
            break;
          case 'apply_translation':
            $result=$this->ajax_apply_translation();
            break;
          case 'delete_key':
            $result=$this->ajax_delete_key();
            break;
          case 'langs_list':
            $result=$this->ajax_langs_list();
            break;
          case 'languages_list': 
            $result=$this->ajax_get_languages_list();
            break;
          case 'export_phase0': 
            $result=$this->ajax_export_phase0();
            break;
          case 'export_phase1':
            $result=$this->ajax_export_phase1();
            break;
          case 'get_key_nfo':
            $result=$this->ajax_get_key_nfo();
            break;
          case 'translation_compare_list':
            $result=$this->ajax_translation_compare_list();
            break;
          case 'translation_compare_update':
            $result=$this->ajax_translation_compare_update();
            break;
          case 'erase':
            $result=$this->ajax_erase();
            break;
          case 'switch_tmode':
            $result=$this->ajax_switch_translator_mode();
            break;
          case 'analyze_list':
            $result=$this->ajax_analyze_list();
            break;
          case 'analyze_detail':
            $result=$this->ajax_analyze_detail();
            break;
          case 'analyze_upgrade':
            $result=$this->ajax_analyze_upgrade();
            break;
        }
        exit($result);
      }
    }



    /* ---------------------------------------------------------------------------
      status functions
    --------------------------------------------------------------------------- */


    /* 
      manage display of translator actions :
       - list
    */
    function display_status_page()
    {
      //default action if not specified into URL's page
      if(!isset($_REQUEST['action']))
      {
        $_REQUEST['action']='list';
      }

      switch($_REQUEST['action'])
      {
        case 'list':
              $this->display_list();
              break;
      }
    }

    /*
      display list page of translation and allow to :
        - view a language
        - modify a language
        - create a new language
    */
    function display_list()
    {
      global $template, $user, $page;
      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_list.tpl');

      if(!isset($_REQUEST['do_scan']))
      {
        $_REQUEST['do_scan']='n';
      }

      //make a scan of directories
      if(($this->my_config['autoscan']=='y') ||
         ($_REQUEST['do_scan']=='y'))
      {
        $disk_filesdatas=array();
        $files=$this->search_languages_files(PHPWG_ROOT_PATH."/language", $this->my_config["charset"]);
        $files=array_merge($files, $this->search_languages_files(PHPWG_ROOT_PATH."/plugins", $this->my_config["charset"]));
        foreach($files as $file)
        {
          $nfo=$this->analyze_filename($file);
          $disk_filesdatas[$nfo['filename']]=$nfo;
        }

        $base_datasfiles=$this->get_all_files();
        if(count($base_datasfiles)!=count($disk_filesdatas))
        {
          $alert_changes=true;
        }
        else
        {
          $alert_changes=false;
        }
        if(!$alert_changes)
        {
          foreach($base_datasfiles as $file)
          {
            if(isset($disk_filesdatas[$file['filename']]))
            {
              if(date('Y-m-d H:i:s', $disk_filesdatas[$file['filename']]['modified'])!=$file['file_date'])
              {
                $alert_changes=true;
              }
            }
            else
            {
              $alert_changes=true;
            }
          }
        }

        if($alert_changes)
        {
          $tabsheet_url="<i><a href='".$this->page_link."&f_tabsheet=maintenance&action=analyze' style='color:#000000;'>".l10n('g001_maintenance')."/".l10n('g001_analyze')."</a></i>";
          array_push($page['infos'], sprintf(l10n('g001_alert_changes_on_files'), $tabsheet_url));
        }
      }

      $langs=$this->get_translated_languages();
      foreach($langs as $val)
      {
        $template->assign_block_vars(
          'lang_header_cols',
          array(
            'WIDTH' => $this->my_config['bars_width']+4,
            'LABEL' => $val
          )
        );
      }

      $nb=0;
      $filelist=$this->make_files_list();
      foreach($filelist as $file => $nfo)
      {
        $nb++;
        $filenfo=explode(':', $file);
        $template->assign_block_vars(
          'files',
          array(
            'TYPE' => $filenfo[0],
            'NAME' => $filenfo[1],
            'FILE' => $filenfo[2],
            'ID' => $nb
          )
        );
        foreach($nfo as $key => $val)
        {
          if(count($val)>0)
          {
            $text = '';
            $width_tt = ceil($this->my_config['bars_width']*$val['nb_to_translate']/$val['nb_elem']);
            $width_tv = ceil($this->my_config['bars_width']*$val['nb_to_validate']/$val['nb_elem']);
            $width_to = ceil($this->my_config['bars_width']*$val['nb_ok']/$val['nb_elem']);
            $width_td = ceil($this->my_config['bars_width']*$val['nb_deleted']/$val['nb_elem']);
          }
          else
          {
            $text = l10n('g001_n/a');
            $width_tt = 0;
            $width_to = 0;
            $width_tv = 0;
            $width_td = 0;
          }
          $template->assign_block_vars(
            'files.nfo_lang',
            array(
              'WIDTH_TT' => $width_tt,
              'WIDTH_TO' => $width_to,
              'WIDTH_TV' => $width_tv,
              'WIDTH_TD' => $width_td,
              'NOTHING' => $text
            )
          );
        }
      }

      $template->assign_vars(
        array(
          "TRANSLATOR_TITLE" => l10n('g001_title_status'),
          //"LNK_CREATE" => $this->page_link.'&f_tabsheet=status&action=create',
          "AJAX_URL_LIST_DETAIL" => $this->page_link.'&ajaxfct=translation_detail_list',
          "AJAX_URL_LANGS_LIST" => $this->page_link.'&ajaxfct=langs_list&fileid=',
          "TRANSLATOR_CHARSET" => substr($user['language'],6),
        )
      );


      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'body_page');
    }






    /* ---------------------------------------------------------------------------
      maintenance functions
    --------------------------------------------------------------------------- */

    /* 
      manage display of maintenance actions :
        - export
        - erase
    */
    function display_maintenance_page()
    {
      global $template;

      //default action if not specified into URL's page
      if(!isset($_REQUEST['action']))
      {
        $_REQUEST['action']='export';
      }

      $template->set_filename('header_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance.tpl');

      $tabsheets=new tabsheets('MAINTENANCE_TAB');
      $tabsheets->add('export',
                            'g001_export',
                            $this->page_link."&f_tabsheet=maintenance&action=export", ($_REQUEST['action']=='export')?true:false);
      $tabsheets->add('erase',
                            'g001_erase',
                            $this->page_link."&f_tabsheet=maintenance&action=erase", ($_REQUEST['action']=='erase')?true:false);
      $tabsheets->add('analyze',
                            'g001_analyze',
                            $this->page_link."&f_tabsheet=maintenance&action=analyze", ($_REQUEST['action']=='analyze')?true:false);
      $tabsheets->make_tabsheets();
      $selected_tab=$tabsheets->get_selected();

      $template->assign_vars(
        array(
          'TRANSLATOR_TITLE' => l10n('g001_maintenance_title').' ['.l10n($selected_tab['caption']).']'
        )
      );

      switch($_REQUEST['action'])
      {
        case 'export':  //esport function
              $this->display_export_page();
              break;
        case 'erase':  //prune deleted&older items 
              $this->display_erase_page();
              break;
        case 'analyze':  //analyze difference between files&database
              $this->display_analyze_page();
              break;
      }

      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'header_page');
    }

    /* 
      export functionnality
    */
    function display_export_page()
    {
      global $template;

      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance_export.tpl');

      $template->assign_vars(
        array(
          'AJAX_URL_EXPORT_PHASE0' => $this->page_link.'&ajaxfct=export_phase0',
          'AJAX_URL_EXPORT_PHASE1' => $this->page_link.'&ajaxfct=export_phase1',
          'PROGRESS_BAR_MAX' => $this->my_config['progress_bar_width'],
          'TRANSLATOR_CHARSET' => $this->my_config["charset"]
        )
      );

      $filelist=$this->get_all_files();
      foreach($filelist as $file => $nfo)
      {
        $nfo_width_tt = ceil(0.75*$this->my_config['bars_width']*$nfo['nb_to_translate']/$nfo['nb_elem']);
        $nfo_width_tv = ceil(0.75*$this->my_config['bars_width']*$nfo['nb_to_validate']/$nfo['nb_elem']);
        $nfo_width_to = ceil(0.75*$this->my_config['bars_width']*$nfo['nb_ok']/$nfo['nb_elem']);
        $nfo_width_td = ceil(0.75*$this->my_config['bars_width']*$nfo['nb_deleted']/$nfo['nb_elem']);

        $template->assign_block_vars(
          'files_row',
          array(
            'TYPE' => $nfo['type'],
            'NAME' => $nfo['name'],
            'FILE' => $nfo['file'],
            'ID' => $nfo['id'],
            'LANG' => $nfo['language'],
            'NFO_WIDTH_TT' => $nfo_width_tt,
            'NFO_WIDTH_TV' => $nfo_width_tv,
            'NFO_WIDTH_TO' => $nfo_width_to,
            'NFO_WIDTH_TD' => $nfo_width_td
          )
        );
      }

      $template->assign_var_from_handle('MAINTENANCE_BODY_PAGE', 'body_page');
    }

    /* 
      erase  functionnality
    */
    function display_erase_page()
    {
      global $template;

      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance_erase.tpl');

      $template->assign_vars(
        array(
          'AJAX_URL_ERASE' => $this->page_link.'&ajaxfct=erase',
          'TRANSLATOR_CHARSET' => $this->my_config["charset"]
        )
      );

      $template->assign_var_from_handle('MAINTENANCE_BODY_PAGE', 'body_page');
    }

   /* 
      analyze functionnality
    */
    function display_analyze_page()
    {
      global $template;

      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance_analyze.tpl');


      $template->assign_vars(
        array(
          'AJAX_URL_DIFF_LIST' => $this->page_link.'&ajaxfct=analyze_list',
          'AJAX_URL_DIFF_DETAIL' => $this->page_link.'&ajaxfct=analyze_detail',
          'AJAX_URL_DIFF_UPGRADE' => $this->page_link.'&ajaxfct=analyze_upgrade',
          'TRANSLATOR_CHARSET' => $this->my_config["charset"]
        )
      );

      $template->assign_var_from_handle('MAINTENANCE_BODY_PAGE', 'body_page');
    }


    /* ---------------------------------------------------------------------------
      languages functions
    --------------------------------------------------------------------------- */
    /* 
      manage langs actions :
       - list languages
       - edit a lang
       - create a new lang
       - delete a lang
    */
    function display_langs_page()
    {
      global $page;

      //default action if not specified into URL's page
      if(!isset($_REQUEST['action']))
      {
        $_REQUEST['action']='list';
      }

      if(isset($_POST['submit']))
      {
        if($_REQUEST['action']=='edit')
        {
          if($this->update_language($_POST))
          {
            $_REQUEST['action']='list';
            array_push($page['infos'], l10n('g001_modify_lang_ok'));
          }
          else
          {
            array_push($page['errors'], l10n('g001_modify_lang_ko'));
          }
        }

        if($_REQUEST['action']=='add')
        {
          if($this->add_language($_POST))
          {
            $_REQUEST['action']='list';
            array_push($page['infos'], l10n('g001_add_lang_ok'));
          }
          else
          {
            array_push($page['errors'], l10n('g001_add_lang_ko'));
          }
        }
      }

      if(isset($_POST['submit_delete']))
      {
        if($this->delete_language($_POST))
        {
          $_REQUEST['action']='list';
          array_push($page['infos'], l10n('g001_delete_lang_ok'));
        }
        else
        {
          array_push($page['errors'], l10n('g001_delete_lang_ko'));
        }
      }

      switch($_REQUEST['action'])
      {
        case 'list':
              $this->display_list_languages();
              break;
        case 'edit':
              $this->display_edit_languages('edit');
              break;
        case 'add':
              $this->display_edit_languages('add');
              break;
      }
    }

    /*
      display a list of existing languages
    */
    function display_list_languages()
    {
      global $template, $user;
      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_langs_global.tpl');

      $template->assign_vars(
        array(
          'TRANSLATOR_TITLE' => l10n('g001_langs_title'),
          'AJAX_URL_LANGUAGES_LIST' => $this->page_link.'&ajaxfct=languages_list',
          'URL_ADD_LANG' => $this->page_link.'&f_tabsheet=langs&action=add'
        )
      );

      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'body_page');
    }

    /*
      display edit page for languages
    */
    function display_edit_languages($mode = 'edit')
    {
      global $template, $user, $page;
      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_langs_add.tpl');

      $lang_nfo=array();
      if($mode=='edit')
      {
        $lang_nfo=$this->get_language_info_by_code($_REQUEST['code']);
        if(count($lang_nfo)==0)
        {
          array_push($page['errors'], l10n('g001_cannot_find_lang'));
        }

        $template->assign_block_vars(
          'edit_mode',
          array(
            'COUNTRY_CODE' => $lang_nfo['country_code'],
            'LANG_CODE' => $lang_nfo['lang_code'],
            'CHARSET' => $lang_nfo['charset'],
            'NBFILE' => $lang_nfo['nbfile']
          )
        );

        //button not disabled, but later may be have to add a confirm message dialog box
        //if($lang_nfo['nbfile']==0)
        {
          $template->assign_block_vars('allow_delete', array());
        }
      }
      else
      {
        if(isset($_REQUEST['langcode']))
        {
          $lang_nfo=array(
            'lang_code' => substr($_REQUEST['langcode'],0,2),
            'name' => '',
            'country_code' => substr($_REQUEST['langcode'],3,2),
            'country' => '',
            'charset' => substr($_REQUEST['langcode'],6),
            'direction' => 'ltr',
            'zero_plural' => 'n'
          );
        }
      }

      if(count($lang_nfo)==0)
      {
        $lang_nfo=array(
          'lang_code' => 'fr',
          'name' => '',
          'country_code' => '',
          'country' => '',
          'charset' => $this->my_config['charset'],
          'direction' => 'ltr',
          'zero_plural' => 'n'
        );
      }

      $template->assign_vars(
        array(
          'TRANSLATOR_TITLE' => l10n('g001_langs_title_edit_'.$mode),
          'COUNTRY_CODE' => $lang_nfo['country_code'],
          'COUNTRY' => $lang_nfo['country'],
          'NAME' => $lang_nfo['name'],
          'DISABLED' => ($mode=='edit')?' disabled':''
        )
      );



      $prev='';
      $langs=$this->get_all_langs_list();
      foreach($langs as $lang)
      {
        if($prev!=$lang['familly'])
        {
          $prev=$lang['familly'];
          $template->assign_block_vars(
            'langs_row',
            array(
              'VALUE' => '',
              'SELECTED' => 'style="font-weight:bold;" disabled',
              'TEXT' => '[-- '.$lang['familly'].' --]'
            )
          );
        }

        $template->assign_block_vars(
          'langs_row',
          array(
            'VALUE' => $lang['code'],
            'SELECTED' => (strtolower($lang_nfo['lang_code'])==strtolower($lang['code']))?' selected':'',
            'TEXT' => '('.$lang['code'].') '.$lang['name']
          )
        );
      }

      $charset=array('iso-8859-1'); //removed utf-8 
      foreach($charset as $val)
      {
        $template->assign_block_vars(
          'charset_row',
          array(
            'VALUE' => $val,
            'SELECTED' => (strtolower($lang_nfo['charset'])==$val)?' selected':'',
            'TEXT' => $val
          )
        );
      }

      $dirs=array('ltr', 'rtl');
      foreach($dirs as $val)
      {
        $template->assign_block_vars(
          'direction_row',
          array(
            'VALUE' => $val,
            'SELECTED' => ($lang_nfo['direction']==$val)?' selected':'',
            'TEXT' => l10n('g001_'.$val)
          )
        );
      }

      $yesno=array('y', 'n');
      foreach($yesno as $val)
      {
        $template->assign_block_vars(
          'zero_plural_row',
          array(
            'VALUE' => $val,
            'SELECTED' => ($lang_nfo['zero_plural']==$val)?' selected':'',
            'TEXT' => l10n('g001_yesno_'.$val)
          )
        );
      }



      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'body_page');
    }


    /* ---------------------------------------------------------------------------
      translation edit functions
    --------------------------------------------------------------------------- */

    /* 
      manage editing actions :
       - compare translation
       - edit a translation
       - create a new language from another (translate to)
    */
    function display_editing_page()
    {
      //default action if not specified into URL's page
      if(!isset($_REQUEST['action']))
      {
        $_REQUEST['action']='none';
      }

      switch($_REQUEST['action'])
      {
        case 'none':
              //$this->display_unknow_action();
              break;
        case 'compare_lang':
              $this->display_comparefiles_action($_REQUEST['fname'],$_REQUEST['ftype'],$_REQUEST['ffile']);
              break;
        case 'edit':
              $this->display_edit_action($_REQUEST['fileid']);
              break;
        case 'editkey':
              $this->display_editkey_action($_REQUEST['key']);
              break;
        case 'translateto':
              $this->display_translateto_action($_REQUEST['fileid']);
              break;
      }
    }

    /*
      display translation edit page of a file
    */
    function display_edit_action($file_id, $page=1, $displaykey='')
    {
      global $template, $user;
      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_edit_global.tpl');

      $nfo=$this->get_file_info_by_id($file_id);
      $langs='';
      foreach($nfo['langs'] as $lang)
      {
        $langs.=$lang['language'].'<br/>';
      }

      foreach($this->statuslist as $key => $val)
      {
        $template->assign_block_vars(
          'status_filters',
          array(
            'VALUE' => $key,
            'SELECT' => ($key==$this->my_config['default_status_filter'])?' selected':'',
            'TEXT' => l10n($val)
          )
        );
      }

      $charset=explode('.', $nfo['language']);
      $template->assign_vars(
        array(
          'NFO_TYPE' => $nfo['type'],
          'NFO_NAME' => $nfo['name'],
          'NFO_VERSION' => $nfo['version'],
          'NFO_FILE' => $nfo['file'],
          'NFO_LANGS' => $langs,
          'NFO_LANG' => $nfo['language'],
          'AJAX_URL_FILE_NFO' => $this->page_link.'&ajaxfct=file_nfo&fileid='.$file_id,
          'AJAX_URL_EDIT_LIST' => $this->page_link.'&ajaxfct=load_edit_list&fileid='.$file_id.'&fpage=',
          'AJAX_URL_EDIT_LIST_DETAIL' => $this->page_link.'&ajaxfct=key_nfo&fileid='.$file_id.'&fkey=',
          'AJAX_URL_TRANSLATION_MODEL' => $this->page_link.'&ajaxfct=key_value&translatelnk=y&fto='.substr($nfo['language'], 0, 2).'&fileid=',
          'AJAX_URL_DO_TRANSLATION' => $this->page_link.'&ajaxfct=do_translation&translatelnk=y&fto='.substr($nfo['language'], 0, 2).'&fileid=',
          'AJAX_URL_VALIDATE_TRANSLATION' => $this->page_link.'&ajaxfct=validate_translation&fileid='.$file_id.'&fkey=',
          'AJAX_URL_VALIDATE_ALL' => $this->page_link.'&ajaxfct=validate_all&fileid='.$file_id,
          'AJAX_URL_APPLY_TRANSLATION' => $this->page_link.'&ajaxfct=apply_translation&fileid='.$file_id.'&fkey=',
          'AJAX_URL_DELETE_KEY' => $this->page_link.'&ajaxfct=delete_key&fileid='.$file_id.'&fkey=',
          'TRANSLATOR_CHARSET' =>  $charset[1], //substr($user['language'],6),
          'TRANSLATOR_TITLE' => sprintf(l10n('g001_file_translate_gui'), $nfo['name'].'/'.$nfo['file'].'/'.$nfo['language'], l10n('g001_file_translate_gui_editkey')),
          'DISPLAYKEY' => $displaykey,
          'PAGE' => $page
        )
      );

      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'body_page');
    }


    /*
      display translation edit page of a file
    */
    function display_editkey_action($uindex)
    {
      $nfo=$this->get_key_nfo_by_unique_index($uindex);
      $page=$this->find_key_page($nfo['id'], $nfo['file_id']);
      $this->display_edit_action($nfo['file_id'], $page, $nfo['id']);
    }

    /*
      display compare page of a file
    */
    function display_comparefiles_action($name, $type, $file)
    {
      global $template, $user;

      $template->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_edit_compare.tpl');

      $template->assign_vars(
        array(
          'AJAX_URL_EDIT_COMPARE_LIST' => $this->page_link.'&ajaxfct=translation_compare_list&fname='.$name.'&ftype='.$type.'&ffile='.$file,
          'AJAX_URL_EDIT_COMPARE_UPDATE' => $this->page_link.'&ajaxfct=translation_compare_update&fname='.$name.'&ftype='.$type.'&ffile='.$file,
          'TRANSLATOR_CHARSET' =>  substr($user['language'],6),
          'NFO_TYPE' => $type,
          'NFO_NAME' => $name,
          'NFO_FILE' => $file,
          'TRANSLATOR_TITLE' => sprintf(l10n('g001_file_translate_gui'), $name."/".$file, l10n("g001_file_translate_gui_compare"))
        )
      );

      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'body_page');
    }

    



    /*
      display create new translation page of a file
    */
    function display_translateto_action($file_id)
    {
      $result=$this->duplicate_file($file_id, $_REQUEST['flang']);
      if($result>0)
      {
        $this->display_edit_action($result);
      }
      else
      {

      }
    }


    /* ---------------------------------------------------------------------------
      config functions
    --------------------------------------------------------------------------- */




    /*
      manage display of config page & save config
    */
    function display_config_page()
    {
      $languages=get_languages();
      if(isset($_POST['submit_save_config']))
      {
        if(!$this->adviser_abort())
        {
          // have to take a look about this code, keeped for reminding until rewritted
          foreach($this->my_config as $key => $val)
          {
            if(isset($_REQUEST['f_translator_'.$key]))
            {
              $this->my_config[$key] = $_REQUEST['f_translator_'.$key];
            }

          }
          $this->display_result(l10n('g001_save_config'), $this->save_config());
        
        }
      }
      $this->display_config();
    }


    /*
      display config page
    */
    function display_config()
    {
      global $template;

      $template->set_filename('body_page',
                  dirname($this->filelocation).'/admin/plugin_admin_config.tpl');



      $template->assign_vars(
        array(
          "TRANSLATOR_TITLE" => l10n('g001_title_config'),
          "TRANSLATOR_CFG_BARS_WIDTH" => $this->my_config['bars_width'],
          "TRANSLATOR_CFG_COLOR_BARS4" => $this->my_config['color_bars4'],
          "TRANSLATOR_CFG_COLOR_BARS3" => $this->my_config['color_bars3'],
          "TRANSLATOR_CFG_COLOR_BARS2" => $this->my_config['color_bars2'],
          "TRANSLATOR_CFG_COLOR_BARS1" => $this->my_config['color_bars1'],
          "TRANSLATOR_CFG_MOUSEOVERCOLOR" => $this->my_config['mouse_over_color'],
          "TRANSLATOR_CFG_NUMBER_OF_ITEMS" => $this->my_config['number_of_items'],
          "TRANSLATOR_CFG_EXPORT_MAXIMUM_ITEMS" => $this->my_config['export_maximum_items']
        )
      );

      foreach($this->statuslist as $key => $val)
      {
        $template->assign_block_vars(
          'default_status_filter_row',
          array(
            'VALUE' => $key,
            'SELECT' => ($key==$this->my_config['default_status_filter'])?' selected':'',
            'TEXT' => l10n($val)
          )
        );
      }        

      $yesno=array('y', 'n');
      foreach($yesno as $key => $val)
      {
        $template->assign_block_vars(
          'autoscan_row',
          array(
            'VALUE' => $val,
            'SELECT' => ($val==$this->my_config['autoscan'])?' selected':'',
            'TEXT' => l10n('g001_yesno_'.$val)
          )
        );
      }        


      $template->assign_var_from_handle('TRANSLATOR_BODY_PAGE', 'body_page');
    }




    /* ---------------------------------------------------------------------------
      unclassed functions
    --------------------------------------------------------------------------- */


    /*
      manage adviser profile
        return true if user is adviser
    */
    function adviser_abort()
    {
      if(is_adviser())
      {
        $this->display_result(l10n("g001_translator_adviser_not_allowed"), false);
        return(true);
      }
      return(false);
    }








    /* ---------------------------------------------------------------------------
      base functions
    --------------------------------------------------------------------------- */

    /*
      make sql column for compare list request (display_comparefiles_action function)
    */
    function make_compare_columns($num, $nbcolumn)
    {
      $returned = array();
      for($i=0;$i<$nbcolumn;$i++)
      {
        if($i==$num)
        {
          array_push($returned, "status as fileid".$i);
        }
        else
        {
          array_push($returned, "'' as fileid".$i);
        }
      }
      return (implode(",", $returned));
    }


    /*
      make a list of array item
    */
    function make_compare_listfile($i, $list)
    {
      $returned=array();
      for($k=0;$k<count($list);$k++)
      {
        array_push($returned, $list[$k][0]);
      }
      $returned = array_splice($returned, $i, 1);
      return (implode(",", $returned));
    }


    /*
      load a $lang file, and give an array of strings
    */
    function load_lang_file($filename)
    {
      $returned=array();
      $fhandle=fopen($filename, 'r');
      if($fhandle)
      {
        $data=fread($fhandle, filesize($filename));
        fclose($fhandle);
      }
    }


    /*
      duplicate a file : create all keys with status 'not_translated' and returns fileid of new file
    */
    function duplicate_file($file_id, $newlang)
    {
      $sql="INSERT INTO ".$this->tables['files']."
            (type, file, language, version, name, filename, nb_elem)
            SELECT type, file, '".$newlang."' as language, version, name, REPLACE(filename, language, '".$newlang."'), nb_elem
            FROM ".$this->tables['files']."
            WHERE id = '".$file_id."' ";
      $result=pwg_query($sql);
      if($result)
      {
        $newid=mysql_insert_id();
        $sql="INSERT INTO ".$this->tables['keys']."
              (id, file_id, version, date, translator, status, translation, uniqueindex)
              SELECT t0.id, '".$newid."', '1', '".date('Y-m-d H:i:s')."', '1', 'not_translated', t0.translation, MD5(CONCAT(t0.id,'-','".$newid."'))
              FROM ".$this->tables['keys']." t0
              WHERE t0.file_id = '".$file_id."'
                AND version IN
                    (
                      SELECT max(tmp.version) FROM ".$this->tables['keys']." tmp
                      WHERE tmp.id = t0.id
                        AND tmp.file_id = t0.file_id
                      GROUP BY tmp.id
                    )";
        $result=pwg_query($sql);
        if($result)
        {
          $this->update_file_counter($newid);
          return($newid);
        }
      }
      return(0);
    }


    /*
      returns translated languages
    */
    function get_translated_languages($lowercase=false)
    {
      $returned=array();

      $sql="SELECT code FROM ".$this->tables['langs']."
            WHERE charset = '".$this->my_config['charset']."' 
            ORDER BY code";
      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_row($result))
        {
          if($lowercase) { $row[0] = strtolower($row[0]); }
          array_push($returned, $row[0]);
        }
      }
      return($returned);
    }

    /*
      returns translated files
    */
    function get_translated_files()
    {
      $returned=array();
      $sql="SELECT DISTINCT type, name, file FROM ".$this->tables['files']."
            ORDER BY type, name, file";
      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          array_push($returned, $row);
        }
      }
      return($returned);
    }

    /*
      returns all files
    */
    function get_all_files()
    {
      $returned=array();
      $sql="SELECT * FROM ".$this->tables['files']."
            ORDER BY type, name, file, language";
      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          array_push($returned, $row);
        }
      }
      return($returned);
    }


    /*
      returns file list of translation by language
    */
    function make_files_list()
    {
      $returned=array();

      $langs=$this->get_translated_languages();
      $files=$this->get_translated_files();

      foreach($files as $file)
      {
        foreach($langs as $lang)
        {
          $returned[$file['type'].":".$file['name'].":".$file['file']][$lang]=array();
        }
      }

      foreach($langs as $lang)
      { 
        $sql="SELECT * FROM ".$this->tables['files']."
              WHERE language = '".$lang."'
              ORDER BY type, file, version
        ";
        $result=pwg_query($sql);
        if($result)
        {
          while($row=mysql_fetch_assoc($result))
          {
           $returned[$row['type'].":".$row['name'].':'.$row['file']][$lang]= $row;
          }
        }
      }
      return($returned);
    }

    /*
      return nfo about a translated file referenced by his id
    */
    function get_file_info_by_id($id)
    {
      $returned=array();
      $sql="SELECT * FROM ".$this->tables['files']."
            WHERE id = '".$id."'";

      $result=pwg_query($sql);
      if($result)
      {
        $returned=mysql_fetch_assoc($result);
        $returned['langs']=array();

        $sql="SELECT language, id FROM ".$this->tables['files']."
              WHERE file = '".$returned['file']."'
                AND type = '".$returned['type']."'
                AND name = '".$returned['name']."'
              ORDER BY language";
        $result=pwg_query($sql);
        if($result)
        {
          while($row=mysql_fetch_assoc($result))
          {
            array_push($returned['langs'], $row);
          }
        }
      }
      return($returned);
    }


    /*
      return nfo about a translated file
      if $lang not given, return nfo about all languages
    */
    function get_file_info($type, $name, $filename, $lang="")
    {
      $returned=array();
      $sql="SELECT * FROM ".$this->tables['files']."
            WHERE file = '".$filename."'
              AND type = '".$type."'
              AND name = '".$name."' ";
      if($lang!="")
      {
        $sql.=" AND language = '".$lang."' ";
      }
      $sql.="ORDER BY language";
      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          $returned[$row['language']]=$row;
        }
      }
      return($returned);
    }

    /*
      just returns the keys list of a file (associate status of key)
    */
    function get_language_file_key_list($file_id)
    {
      $return=array();
      $sql="SELECT DISTINCT id, status
            FROM ".$this->tables['keys']."
            WHERE file_id = '".$file_id."'";
      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          $return[$row['id']]=$row['status'];
        }
      }
      return($return);
    }

    /*
      update file date time in databse (after export or import...)
      $nfo is an array with 'id' and 'filename' items
    */
    function update_date_file($nfo, $reset=false)
    {
      if($reset)
      {
        $sql="UPDATE ".$this->tables['files']."
              SET file_date='0000-00-00 00:00:00'
              WHERE id=".$nfo['id'];
      }
      else
      {
        $sql="UPDATE ".$this->tables['files']."
              SET file_date=FROM_UNIXTIME(".filemtime($nfo['filename']).")
              WHERE id=".$nfo['id'];
      }

      $result=pwg_query($sql);
      if($result)
      {
        return(true);
      }
      return(false);
    }

    /*
      returns a list (each item is an array) of file's items
      work with $page, number of items per page is given by my_config['number_of_items'] value
      status values :
        '' : no filter
        'ok' 
        'valide_it'
        'not_translated'      
    */
    function get_language_file_items($file_id, $status, $page)
    {
      $returned['list']=array();
      $returned['pages']=1;
      $sql="SELECT SQL_CALC_FOUND_ROWS * FROM ".$this->tables['keys']." t0
            WHERE file_id = '".$file_id."'
              AND version in
            (
              SELECT max(tmp.version) FROM ".$this->tables['keys']." tmp
              WHERE tmp.id = t0.id
                AND tmp.file_id = t0.file_id
              GROUP BY tmp.id
            ) ";

      if($status!='')
      {
        $sql.=" AND status = '".$status."'";
      }
      $sql.=" ORDER by id ASC
              LIMIT ".(($page-1)*$this->my_config['number_of_items']).", ".$this->my_config['number_of_items'];

      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          array_push($returned['list'], $row);
        }

        $sql="select FOUND_ROWS()";
        $result=pwg_query($sql);
        if($result)
        {
          $row=mysql_fetch_row($result);
          $returned['pages']=$row[0];
        }

      }
      return($returned);
    }

    /*
      return page number for a key
    */
    function find_key_page($key, $file_id)
    {
      $returned=1;
      $sql="SELECT id FROM ".$this->tables['keys']." t0
            WHERE file_id = '".$file_id."'
              AND version in
            (
              SELECT max(tmp.version) FROM ".$this->tables['keys']." tmp
              WHERE tmp.id = t0.id
                AND tmp.file_id = t0.file_id
              GROUP BY tmp.id
            )
            ORDER by id ASC";
      $result=pwg_query($sql);
      if($result)
      {
        $list=array();
        $i=1;
        while($row=mysql_fetch_row($result))
        {
          $list[$row[0]]=$i;
          $i++;
        }
        if(isset($list[$key]))
        {
          $returned=ceil($list[$key]/$this->my_config['number_of_items']);
        }
      }
      return($returned);
    }


    /*
      returns information about a key
        first elem : higher version
    */
    function get_key_nfo($fileid, $key)
    {
      $returned=array();
      $sql="SELECT ".$this->tables['keys'].".*, t2.username as validate_by_name, t1.username as create_by_name
            FROM  (".$this->tables['keys']."
                    LEFT OUTER JOIN ".USERS_TABLE." t1
                    ON ".$this->tables['keys'].".translator = t1.id
                  )
                    LEFT OUTER JOIN ".USERS_TABLE." t2
                    ON ".$this->tables['keys'].".validate_by = t2.id
            WHERE ".$this->tables['keys'].".file_id = '".$fileid."'
              AND ".$this->tables['keys'].".id = '".$key."'
            ORDER BY ".$this->tables['keys'].".version DESC";

      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          array_push($returned, $row);
        }
      }
      return($returned);
    }


    /*
      returns information about a key
    */
    function get_key_nfo_by_unique_index($uniqueindex)
    {
      $returned=array();
      $sql="SELECT ".$this->tables['keys'].".*,
                    t2.username as validate_by_name, t1.username as create_by_name,
                    t3.type, t3.file, t3.language, t3.name, t3.file_date
            FROM  (".$this->tables['keys']."
                    LEFT OUTER JOIN ".USERS_TABLE." t1
                    ON ".$this->tables['keys'].".translator = t1.id
                  )
                  LEFT OUTER JOIN ".USERS_TABLE." t2
                  ON ".$this->tables['keys'].".validate_by = t2.id,
                  ".$this->tables['files']." t3
            WHERE ".$this->tables['keys'].".uniqueindex = '".$uniqueindex."'
              AND ".$this->tables['keys'].".file_id = t3.id
            ORDER BY ".$this->tables['keys'].".version DESC LIMIT 0,1 ";

      $result=pwg_query($sql);
      if($result)
      {
        $returned=mysql_fetch_assoc($result);
      }
      return($returned);
    }

    /*
      change status value for a key
    */
    function change_key_status($fileid, $key, $version, $validateby, $status)
    {
      $sql='UPDATE '.$this->tables['keys'].' t0,
                   (SELECT max(t1.version) as maxversion, t1.id, t1.file_id
                    FROM '.$this->tables['keys'].' t1
                    GROUP BY t1.id, t1.file_id
                   ) t2 
            SET t0.status = "'.$status.'",
                t0.validate_by = "'.$validateby.'",
                t0.validate_date = "'.date('Y-m-d H:i:s').'"
            WHERE t0.file_id = "'.$fileid.'" ';

      if($key!='')
      {
        $sql.=' AND t0.id = "'.$key.'" ';
      }
      else
      {
        $sql.=' AND t0.status = "valide_it" ';
      }

      if($version!='')
      {
        $sql.=' AND t0.version = "'.$version.'" ';
      }
      else
      {
        $sql.=' AND t0.version = t2.maxversion
                AND t0.id = t2.id
                AND t0.file_id = t2.file_id';
      }

      $result=pwg_query($sql);
      if($result)
      {
        $result=$this->update_file_counter($fileid);
        $result=pwg_query($sql);
        if($result)
        {
          return(true);
        }
      }
      return(false);
    }

    /*
      insert a new translation for a key
    */
    function insert_key_translation($fileid, $key, $createdby, $value, $status='valide_it')
    {
      $sql="SELECT t1.version, t0.status 
            FROM ".$this->tables['keys']." t0,
                  (
                    SELECT max(version) as version FROM ".$this->tables['keys']."
                    WHERE id = '".$key."'
                      AND file_id = '".$fileid."'
                  ) t1
            WHERE t0.id = '".$key."'
              AND t0.file_id = '".$fileid."'
              AND t0.version = t1.version";
      $result=pwg_query($sql);
      if($result)
      {
        $nextversion=mysql_fetch_row($result);
        if(($nextversion[0]==1)&&($nextversion[1]=='not_translated'))
        {
          $sql="UPDATE ".$this->tables['keys']."
                SET status = '".$status."',
                    translator = '".$createdby."',
                    date = '".date("Y-m-d H:i:s")."',
                    translation = '".addslashes($value)."'
                WHERE id = '".$key."'
                  AND file_id = '".$fileid."'
                  AND version = '1'";
        }
        else//
        {
          $sql="INSERT INTO ".$this->tables['keys']." VALUES(
            '".$key."',
            '".$fileid."',
            '".($nextversion[0]+1)."',
            '".date("Y-m-d H:i:s")."',
            '".$createdby."',
            '".$status."',
            '".addslashes($value)."',
            '0',
            '0000-00-00 00:00:00',
            '".$this->calculate_uniquekey($key, $fileid)."'
          )";
        }
        $result=pwg_query($sql);
        if($result)
        {
          $result=$this->update_file_counter($fileid);
          if($result)
          {
            return(true);
          }
        }
      }
      return(false);
    }


    function delete_key($fileid, $key, $deleted_by)
    {
      $sql="SELECT t1.id 
            FROM (".$this->tables['files']." t1
                  LEFT OUTER JOIN ".$this->tables['files']." t2
                  ON t1.type = t2.type
                  AND t1.file = t2.file
                  AND t1.name=t2.name
                 )
                  LEFT OUTER JOIN ".$this->tables['keys']." t3
                  ON t3.file_id = t1.id
            WHERE t2.id = '".$fileid."'
              AND t3.id = '".$key."';";
      $result=pwg_query($sql);
      if(!$result)
      {
        return(false);
      }
      $fileids=array();
      while($row=mysql_fetch_array($result))
      {
        array_push($fileids, $row[0]);
      }
      
      $sql="UPDATE ".$this->tables['keys']."
            SET status = 'deleted'
            WHERE file_id in (".implode(',', $fileids).")
              AND id = '".$key."'";
      $result=pwg_query($sql);
      if($result)
      {
        $result=$this->update_file_counter($fileids);
        if($result)
        {
          return(true);
        }
      }
      return(false);
    }

    function update_file_counter($fileid='')
    {
      $sql="UPDATE ".$this->tables['files']." tr,
            (SELECT SUM(cok) AS ok,
                    SUM(cvalide_it) AS valide_it,
                    SUM(cto_translate) AS to_translate,
                    SUM(cdeleted) AS deleted,
                    (SUM(cok)+SUM(cvalide_it)+SUM(cto_translate)+SUM(cdeleted)) as nbelem,
                    ts.file_id AS file_id
            FROM (SELECT t0.file_id,
                          IF(status='ok', COUNT(status), 0) AS 'cok',
                          IF(status='valide_it', COUNT(status), 0) AS 'cvalide_it',
                          IF(status='not_translated', COUNT(status), 0) AS 'cto_translate',
                          IF(status='deleted', COUNT(status), 0) AS 'cdeleted'
                  FROM ".$this->tables['keys']." t0
                  WHERE t0.version IN
                                      (
                                        SELECT max(tmp.version)
                                        FROM ".$this->tables['keys']." tmp
                                        WHERE tmp.id = t0.id
                                          AND tmp.file_id = t0.file_id
                                        GROUP BY tmp.id
                                      )
                  GROUP BY file_id,status
                 ) AS ts
            GROUP BY ts.file_id
            ) AS tsum

            SET tr.nb_ok = tsum.ok,
                tr.nb_deleted = tsum.deleted,
                tr.nb_to_validate = tsum.valide_it,
                tr.nb_to_translate = tsum.to_translate,
                tr.nb_elem = tsum.nbelem
            WHERE tsum.file_id = tr.id ";
      if(is_array($fileid))
      {
        $sql.=" and tr.id in (".implode(',', $fileid).")";
      }
      elseif($fileid!='')
      {
        $sql.=" and tr.id = '".$fileid."' ";
      }
      $result=pwg_query($sql);
      if($result)
      {
        return(true);
      }
      return(false);
    }



    /*
      returns langs list from langs table
    */
    function get_langs_list()
    {
      $returned=array();
      $sql="SELECT t0.*,
                    ".$this->tables['langs_codes'].".name as isoname,
                    ".$this->tables['langs_codes'].".familly as isofamilly,
                    count(t1.id) as nbfile
            FROM (".$this->tables['langs']." t0
              LEFT OUTER JOIN ".$this->tables['langs_codes']."
              ON t0.lang_code = ".$this->tables['langs_codes'].".code)
                LEFT OUTER JOIN ".$this->tables['files']." t1
                ON t1.language = t0.code
            GROUP BY t0.code
            ORDER BY ".$this->tables['langs_codes'].".familly,
                     ".$this->tables['langs_codes'].".name ";
      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          $returned[$row['lang_code']]=$row;
        }
      }
      return($returned);
    }

    /*
      returns langs list from langs_code table
    */
    function get_all_langs_list()
    {
      $returned=array();
      $sql="SELECT * FROM ".$this->tables['langs_codes']."
            ORDER BY familly, name";

      $result=pwg_query($sql);
      if($result)
      {
        while($row=mysql_fetch_assoc($result))
        {
          array_push($returned, $row);
        }
      }
      return($returned);
    }

    /*
      return info about a lang referenced into langs table
        search by 'code' (eg. en_UK.iso-8859-1)
    */
    function get_language_info_by_code($code)
    {
      $returned=array();
      $sql="SELECT t1.*, count(t0.id) as nbfile
            FROM ".$this->tables['langs']." t1
              LEFT OUTER JOIN ".$this->tables['files']." t0
              ON t0.language = t1.code
            WHERE t1.code = '".$code."'
            GROUP BY t1.code
            ";

      $result=pwg_query($sql);
      if($result)
      {
        $returned=mysql_fetch_assoc($result);
      }
      return($returned);
    }

    /*
      insert a new lang into 'langs' table
        $nfo is an array
    */
    function add_language($nfo)
    {
      $sql="INSERT INTO ".$this->tables['langs']." VALUES (
        '".strtolower($nfo['f_lang_code'])."',
        '".strtoupper($nfo['f_country_code'])."',
        '".$nfo['f_charset']."',
        '".$nfo['f_direction']."',
        '".$nfo['f_name']."',
        '".$nfo['f_country']."',
        '".$nfo['f_zero_plural']."',
        '".strtolower($nfo['f_lang_code']).'_'.strtoupper($nfo['f_country_code']).'.'.$nfo['f_charset']."'
      )";
      $result=pwg_query($sql);
      if($result)
      {
        return(true);
      }
      return(false);
    }

    /*
      modify a lang from 'langs' table
        $nfo is an array
    */
    function update_language($nfo)
    {
      $sql="UPDATE ".$this->tables['langs']."
            SET direction = '".$nfo['f_direction']."',
                name = '".$nfo['f_name']."',
                country = '".$nfo['f_country']."',
                zero_plural = '".$nfo['f_zero_plural']."'
            WHERE lang_code = '".$nfo['f_lang_code']."'
              AND country_code = '".$nfo['f_country_code']."'
              AND charset = '".$nfo['f_charset']."' ";
      $result=pwg_query($sql);
      if($result)
      {
        return(true);
      }
      return(false);
    }

    /*
      delete a lang from 'langs' table, 'files' table and 'keys' table
        $nfo is an array
    */
    function delete_language($nfo)
    {
      $sql="DELETE FROM ".$this->tables['langs']."
            WHERE lang_code = '".$nfo['f_lang_code']."'
              AND country_code = '".$nfo['f_country_code']."'
              AND charset = '".$nfo['f_charset']."' ";
      $result=pwg_query($sql);
      if($result)
      {
        $language = $nfo['f_lang_code']."_".$nfo['f_country_code'].".".$nfo['f_charset'];
        $sql="DELETE FROM ".$this->tables['keys']."
              WHERE file_id IN
                (SELECT id FROM ".$this->tables['files']." WHERE language='".$language."')";
        $result=pwg_query($sql);
        if($result)
        {
          $sql="DELETE FROM ".$this->tables['files']."
                WHERE language='".$language."'";
          $result=pwg_query($sql);
          if($result)
          {
            return(true);
          }
        }
      }
      return(false);
    }



    /*
      export an file into a .lang.php file
        $option is an array
          translator_export : 'y' or 'n' => if yes, add translator informations
          exported_items : 'ok' or 'valide_it' => if ok, only items with 'ok' status are exported
    */
    function export($fileid, $options=
      array(
        'exported_items' => 'ok',
        'translator_export' => 'n'
      )
    )
    {
      global $user;

      $file_nfo=$this->get_file_info_by_id($fileid);

      $returned=array('errors' => array(), 'nfo'=>$file_nfo);

      $sql="SELECT t0.* FROM ".$this->tables['keys']." t0
            WHERE t0.file_id = '".$fileid."' 
              AND t0.version in
            (
              SELECT max(tmp.version) FROM ".$this->tables['keys']." tmp
              WHERE tmp.id = t0.id
                AND tmp.file_id = t0.file_id
              GROUP BY tmp.id
            )  ";


      if($options['translator_export']=='n')
      {
        $sql.=" AND (t0.status = 'ok'" ;
        if($options['exported_items']=='valide_it')
        {
          $sql.=" OR t0.status = 'valide_it' ";
        }

        $sql.=')';
      }
      $sql.=' ORDER BY t0.id';
      $result=pwg_query($sql);
      if($result)
      {
        $language_info=$this->get_language_info_by_code($file_nfo['language']);
        $lang_nfo=explode('.', $file_nfo['language']);
        $row_name='lang_row';
        if($options['translator_export']=='y')
        {
          $row_name.='_translator';
        }


        $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
        $local_tpl->set_filename('body_page',
                      dirname($this->filelocation).'/admin/plugin_admin_lang_php_'.$lang_nfo[1].'.tpl');

        $nbexported=0;
        while($row=mysql_fetch_array($result))
        {
          switch($row['status'])
          {
            case 'ok':
              $classstatus="tonfo";
              $status=l10n('g001_validated');
              break;
            case 'valide_it':
              $classstatus="tvnfo";
              $status=l10n('g001_to_validate');
              break;
            case 'not_translated':
              $classstatus="ttnfo";
              $status=l10n('g001_to_translate');
              break;
            case 'deleted':
              $classstatus="tdnfo";
              $status=l10n('g001_deleted');
              break;
          }
          $nbexported++;

          if(preg_match("/\s\[\#\d+]/", $row['id'], $tmp)>0)
          {
            //key is an item array
            preg_match("/\d+/", $tmp[0], $tmp);
            $keytab = "[".$tmp[0]."]";
            $tmp = preg_split("/\s\[\#\d+]/", $row['id']);
            $key = $tmp[0];
          }
          else
          {
            $key = $row['id'];
            $keytab = "";
          }

          $local_tpl->assign_block_vars(
            $row_name,
            array(
              'KEY' => $key,
              'KEYTAB' => $keytab,
              'VALUE' => addslashes($row['translation']),
              'STATUSCLASS' => $classstatus,
              'IDNUM' => $fileid*100000+$nbexported,
              'NAMENUM' => $row['uniqueindex'],
              'STATUS' => $status,
              'VERSION' => $row['version'],
              'DATE' => $row['date'],
              'TYPE' => $file_nfo['type'],
              'NAME' => $file_nfo['name'],
              'LANG' => $lang_nfo[0],
              'CHARSET' => $lang_nfo[1],
              'FILENAME' => $file_nfo['file']
            )
          );
          
        }

        if($file_nfo['file']=='common.lang.php')
        {
          $local_tpl->assign_block_vars(
            'lang_info_row',
            array(
              'LANGUAGE_NAME' => $language_info['name'],
              'COUNTRY' => $language_info['country'],
              'CHARSET' => $language_info['charset'],
              'LTR' => $language_info['direction'],
              'CODE' => $language_info['code'],
              'ZERO_PLURAL' => ($language_info['zero_plural']=='y')?'true':'false'
            )
          );
        }

        $local_tpl->assign_vars(
          array(
            'DATE' => date('Y-m-d H:i:s'),
            'LANG' => $lang_nfo[0],
            'CHARSET' => $lang_nfo[1],
            'TYPE' => $file_nfo['type'],
            'NAME' => $file_nfo['name'],
            'TRANSLATOR' => $this->plugin_name,
            'VERSION' => TRANSLATOR_VERSION,
            'NBELEM' => $file_nfo['nb_elem'],
            'NBEXPORTED' => $nbexported,
            'EXPORTEDBY' => $user['username'],
            'EXPORT_TYPE' => $options['exported_items'],
            'TRANSLATOR_EXPORT' => ($options['translator_export']=='y')?'YES':'NO'
          )
        );

        $filename=$file_nfo['filename'];

        $copyok=false;
        if($options['translator_export']=='n')
        {
          if(file_exists($filename))
          {
            if(@copy($filename, $filename.'.backup'))
            {
              $copyok=true;
              
            }
            else
            {
              array_push($returned['errors'], l10n('g001_error_export_2'));
            }
          }
          else
          {
            $copyok=true;
          }
        }
        else
        {
          $filename.="s";
        }

        $returned['nfo']['exported_file']=$filename;
        $returned['nfo']['nbexported']=$nbexported;

        
        if((($options['translator_export']=='n')&&($copyok))||(($options['translator_export']=='y')))
        {
          $this->mkpath(dirname($filename));
          @chmod(dirname($filename), 0777);
          $fhandle=@fopen($filename, 'w');
          if($fhandle)
          {
            fwrite($fhandle, $local_tpl->parse('body_page', true));
            fclose($fhandle);
            chmod($filename, 0777);
            $this->update_date_file($file_nfo);
          }
          else
          {
            array_push($returned['errors'], l10n('g001_error_export_3'));
          }
        }
      }
      else
      {
        array_push($returned['errors'], l10n('g001_error_export_1'));
      }
      return($returned);
    }

    function mkpath($path)
    {
      if(@mkdir($path, 0777) or file_exists($path))
      {
        return(true);
      }
      else
      { 
        return($this->mkpath(dirname($path)) and mkdir($path, 0777));
      }
    }

    /*
      compare keys between server file and database
        return an array of each keys as array key, and value as :
        4 : exist on file, but not in database
        5 : exist in database, but not in file
        key existing in both are not returned
    */
    function compare_file_and_database_keys($ftype, $fname, $ffile, $flang)
    {
      $lang=array();
      $return=array();

      $nfo=$this->get_file_info($ftype, $fname, $ffile, $flang);
      $database_keys=$this->get_language_file_key_list($nfo[$flang]['id']);
      $return['filenfo']=$nfo[$flang];
      $return['keynfo']=array();

      //include .lang.php file to fill array 'lang'
      @include($nfo[$flang]['filename']);

      //check if server file's keys are in database
      foreach($lang as $key => $val)
      {
        if(is_array($val))
        {
          //$val is an array, check if all items of this array exists in database
          while(list($ikey, $ival) = each($val))
          {
            $arraykey=$key." [#".$ikey."]";
            if(!isset($database_keys[$arraykey]))
            {
              //keys doesn't exist in database
              $return['keynfo'][$arraykey]=array('4', $ival);
            }
          }
        }
        else
        {
          if(!isset($database_keys[$key]))
          {
            //keys doesn't exist in database
            $return['keynfo'][$key]=array('4', $val);
          }
        }
      }

      //check if database's keys exists in server file
      foreach($database_keys as $key => $val)
      {
        if(preg_match("/\s\[\#\d+]/", $key, $tmp)>0)
        {
          //key is an item array
          preg_match("/\d+/", $tmp[0], $tmp);
          $keytab = $tmp[0];
          $tmp = preg_split("/\s\[\#\d+]/", $key);
          $testkey = $tmp[0];
          $isset=isset($lang[$testkey][$keytab]);
        }
        else
        {
          $isset=isset($lang[$key]);
        }
        if(!$isset)
        {
          //keys doesn't exist in server file
          if($val!='deleted')
          {
            //if key status is not deleted, add the key 
            $return['keynfo'][$key]=array('5', '');
          }
        }
      }
      return($return);
    }

    function erase_file_from_database($file_id)
    {
      $sql="DELETE FROM ".$this->tables['files']." WHERE id='".$file_id."'";
      if(pwg_query($sql))
      {
        $sql="DELETE FROM ".$this->tables['keys']." WHERE file_id='".$file_id."'";
        if(pwg_query($sql))
        {
          return(true);
        }
      }
      return(false);
    }

    /* ---------------------------------------------------------------------------
      ajax functions
    --------------------------------------------------------------------------- */

    /*
      get detail information for status list
    */
    function ajax_translation_detail_list()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_detail.tpl');

      $langs=$this->get_translated_languages();
      $nfo=$this->get_file_info($_REQUEST['ftype'], $_REQUEST['fname'], $_REQUEST['ffile']);


      $nbtranslatedlang=0;
      $text="";
      $text2="";
      foreach($langs as $lang)
      {        
        if(isset($nfo[$lang]))
        {
          $nbtranslatedlang++;
          $local_tpl->set_filename('list_detail',
              dirname($this->filelocation).'/admin/plugin_admin_detail_1.tpl');

          $local_tpl->assign_vars(
            array(
              'NBELEM' => $nfo[$lang]['nb_elem'],
              'NBELEMTT' => $nfo[$lang]['nb_to_translate'],
              'NBELEMTV' => $nfo[$lang]['nb_to_validate'],
              'NBELEMTO' => $nfo[$lang]['nb_ok'],
              'NBELEMTD' => $nfo[$lang]['nb_deleted'],
              'PCTTT' => round(100*$nfo[$lang]['nb_to_translate']/$nfo[$lang]['nb_elem'],2),
              'PCTTV' => round(100*$nfo[$lang]['nb_to_validate']/$nfo[$lang]['nb_elem'],2),
              'PCTTO' => round(100*$nfo[$lang]['nb_ok']/$nfo[$lang]['nb_elem'],2),
              'PCTTD' => round(100*$nfo[$lang]['nb_deleted']/$nfo[$lang]['nb_elem'],2)
            )
          );

          $text_tmp='<a href="'.$this->page_link.'&f_tabsheet=editing&action=edit&fileid='.$nfo[$lang]['id'].'">'.l10n('g001_edit').'</a><br/>
                     <a style="cursor:pointer;position:absolute;" id="imenu1'.$nfo[$lang]['id'].'" onclick="display_menu1(\''.$nfo[$lang]['id'].'\')">'.l10n('g001_translate_to').'</a>';

          /* <a href="'.$this->page_link.'&f_tabsheet=tools&action=translateto&fileid='.$nfo[$lang]['id'].'">'.l10n('g001_translate_to').'</a>'; */


        }
        else
        {
          $local_tpl->set_filename('list_detail',
              dirname($this->filelocation).'/admin/plugin_admin_detail_2.tpl');
          $local_tpl->assign_vars(
            array(
              'CELLTEXT' => l10n('g001_no_translation_exists')
            )
          );
    
          $text_tmp='<br/>&nbsp;';
        }
        $text.=$local_tpl->parse('list_detail', true);
                    
        $local_tpl->set_filename('list_detail2',
          dirname($this->filelocation).'/admin/plugin_admin_detail_2.tpl');
          $local_tpl->assign_vars(
            array(
              'CELLTEXT' => $text_tmp
            )
          );
        $text2.=$local_tpl->parse('list_detail2', true);

      }
      if($nbtranslatedlang>1)
      {
        $local_tpl->assign_block_vars(
          'can_compare',
          array(
            'COMPARE_LANGUAGE_LINK' => $this->page_link.'&f_tabsheet=editing&action=compare_lang&ftype='.$_REQUEST['ftype'].
                                      '&fname='.urlencode($_REQUEST['fname']).
                                      '&ffile='.urlencode($_REQUEST['ffile'])
          )
        );
      }
      else
        $local_tpl->assign_block_vars(
          'cannot_compare',
          array(
          )
        );


      $local_tpl->assign_vars(
        array(
          'TEXT' => $text,
          'TEXT2' => $text2,
        )
      );

      return($local_tpl->parse('body_page', true));
    }


    /*
      get file detail nfo 
    */
    function ajax_file_nfo()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_edit_detail1.tpl');

      $nfo=$this->get_file_info_by_id($_REQUEST['fileid']);

      $local_tpl->set_filename('list_detail',
          dirname($this->filelocation).'/admin/plugin_admin_detail_1.tpl');


      $local_tpl->assign_vars(
        array(
          'NBELEM' => $nfo['nb_elem'],
          'NBELEMTT' => $nfo['nb_to_translate'],
          'NBELEMTV' => $nfo['nb_to_validate'],
          'NBELEMTO' => $nfo['nb_ok'],
          'NBELEMTD' => $nfo['nb_deleted'],
          'PCTTT' => round(100*$nfo['nb_to_translate']/$nfo['nb_elem'],2),
          'PCTTV' => round(100*$nfo['nb_to_validate']/$nfo['nb_elem'],2),
          'PCTTO' => round(100*$nfo['nb_ok']/$nfo['nb_elem'],2),
          'PCTTD' => round(100*$nfo['nb_deleted']/$nfo['nb_elem'],2)
        )
      );

      $nfo_work_progress=$local_tpl->parse('list_detail', true);

      $nfo_width_tt = ceil(2.5*$this->my_config['bars_width']*$nfo['nb_to_translate']/$nfo['nb_elem']);
      $nfo_width_tv = ceil(2.5*$this->my_config['bars_width']*$nfo['nb_to_validate']/$nfo['nb_elem']);
      $nfo_width_to = ceil(2.5*$this->my_config['bars_width']*$nfo['nb_ok']/$nfo['nb_elem']);
      $nfo_width_td = ceil(2.5*$this->my_config['bars_width']*$nfo['nb_deleted']/$nfo['nb_elem']);


      $local_tpl->assign_vars(
        array(
          'NFO_WIDTH_TT' => $nfo_width_tt,
          'NFO_WIDTH_TV' => $nfo_width_tv,
          'NFO_WIDTH_TO' => $nfo_width_to,
          'NFO_WIDTH_TD' => $nfo_width_td,
          'NFO_WORK_PROGRESS' => $nfo_work_progress,
          'NFO_LNK_ACTION_VALIDATE_ALL' => ($nfo['nb_to_validate']>0)?'['.l10n('g001_validate_all').']':''
        )
      );

      return($local_tpl->parse('body_page', true));
    }

    /*
      load list of items
    */
    function ajax_load_edit_list()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_edit_detail2.tpl');

      $file_id=$_REQUEST['fileid'];
      $status='';
      $page=1;
      if(isset($_REQUEST['fstatus']))
      {
        $status=$_REQUEST['fstatus'];
      }
      if(isset($_REQUEST['fpage']))
      {
        $page=$_REQUEST['fpage'];
      }
      if($status=='none')
      {
        $status='';
      }

      $fileitems=$this->get_language_file_items($file_id, $status, $page);
      //$nfo=$this->get_file_info_by_id($file_id);

      foreach($fileitems['list'] as $item)
      {
        switch($item['status'])
        {
          case 'ok':
            $status="g001_graphbar3";
            break;
          case 'valide_it':
            $status="g001_graphbar2";
            break;
          case 'not_translated':
            $status="g001_graphbar1";
            break;
          case 'deleted':
            $status="g001_graphbar4";
            break;
        }

        $local_tpl->assign_block_vars(
          'keys',
          array(
            'KEY' => $item['id'],
            'STATUS' => $status,
            'VALUE' => $item['translation']
          )
        );
      }

      $pagesnav=new pages_navigation();
      $pagesnav->set_nb_items($fileitems['pages']);
      $pagesnav->set_nb_items_per_page($this->my_config['number_of_items']);
      $pagesnav->set_current_page($page);

      $local_tpl->assign_var('PAGES_NAVIGATION', $pagesnav->make_navigation('load_list'));

      return($local_tpl->parse('body_page', true));
    }

    /*
      return keys nfo in an html formatted string
    */
    function ajax_key_nfo()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_edit_detail3.tpl');

      $idkey=$_REQUEST['fkey'];
      $nfokey=$this->get_key_nfo($_REQUEST['fileid'], $idkey);
      $nfofile=$this->get_file_info_by_id($_REQUEST['fileid']);

      $local_tpl->assign_vars(
        array(
          'TEXT' => $nfokey[0]['translation'],
          'ID' => $idkey,
          'VERSION' => $nfokey[0]['version'],
        )
      );

      if(($nfokey[0]['status']=='valide_it')||($nfokey[0]['status']=='deleted'))
      {
        $local_tpl->assign_block_vars(
          'valide_it', array()
        );
      }
      if($nfokey[0]['status']!='deleted')
      {
        $local_tpl->assign_block_vars(
          'delete_it', array()
        );
      }

      foreach($nfofile['langs'] as $nfo)
      {
        $local_tpl->assign_block_vars(
          'langs',
          array(
            'LANG' => $nfo['id'],
            'SELECT' => ($nfo['language']==$this->my_config['mainlang'].'.'.$this->my_config['charset'])?' selected':'',
            'TEXT' => $nfo['language']
          )
        );
      }

      foreach($nfokey as $key =>$nfo)
      {
        if($key==0)
        {
          $local_tpl->assign_block_vars(
            'versions_head', array()
          );
        }

        $local_tpl->assign_block_vars(
          'versions_rows',
          array(
            'ID' => $idkey,
            'VERSION' => $nfo['version'],
            'CREATE_DATE' => $nfo['date'],
            'CREATE_BY' => $nfo['create_by_name'],
            'VALIDATE_DATE' => ($nfo['validate_date']=='0000-00-00 00:00:00')?'':$nfo['validate_date'],
            'VALIDATE_BY' => $nfo['validate_by_name'],
            'VALUE' => $nfo['translation']
          )
        );
      }
      return($local_tpl->parse('body_page', true));
    }

    /*
      just return value of a key
    */
    function ajax_key_value()
    {
      $translate = new translate();
      $nfo=$this->get_key_nfo($_REQUEST['fileid'], $_REQUEST['fkey']);

      $returned=$nfo[0]['translation'];

      if(isset($_REQUEST['translatelnk'])&&($_REQUEST['translatelnk']=='y')&&
         (isset($_REQUEST['ffrom']))&& (isset($_REQUEST['fto']) ))
      {
        $class=array(
          'ok' => 'g001_graphbar3',
          'valide_it' => 'g001_graphbar2',
          'not_translated' => 'g001_graphbar1',
          'deleted' => 'g001_graphbar4'
        );
        $returned='<div class="'.$class[$nfo[0]['status']].'" style="width:10px;">&nbsp;</div>'.$returned;
        if($translate->can_translate($_REQUEST['ffrom'], $_REQUEST['fto']))
        {
          $returned.='<br><a style="cursor:pointer;" onclick="do_translation(\''.$nfo[0]['id'].'\')">'.l10n('g001_google_translate').'</a>';
        }
      }

      return($returned);
    }

    /*
      validate a translation
    */
    function ajax_validate_translation()
    {
      global $user;

      $result=$this->change_key_status($_REQUEST['fileid'], $_REQUEST['fkey'], $_REQUEST['fversion'], $user['user_id'], 'ok');
      if($result)
      {
        return('ok');
      }
      else
      {
        return('ko');
      }
    }

    /*
      validate all translation wich status equal 'valide_it'
    */
    function ajax_validate_all()
    {
      global $user;

      $result=$this->change_key_status($_REQUEST['fileid'], '', '', $user['user_id'], 'ok');
      if($result)
      {
        return('ok');
      }
      else
      {
        return('ko');
      }
    }


    /*
      delete a key. key is deleted in all languages.
    */
    function ajax_delete_key()
    {
      global $user;

      $result=$this->delete_key($_REQUEST['fileid'], $_REQUEST['fkey'], $user['user_id']);
      if($result)
      {
        return('ok');
      }
      else
      {
        return('ko');
      }
    }


    /*
      apply a translation
    */
    function ajax_apply_translation()
    {
      global $user;

      $result=$this->insert_key_translation($_REQUEST['fileid'], $_REQUEST['fkey'], $user['user_id'], stripslashes($_POST['fvalue']));
      if($result)
      {
        return('ok');
      }
      else
      {
        return('ko');
      }
    }

    /*
      call google's translation page, analyze it and restitute a script
    */
    function ajax_do_translation()
    {
      global $user;

      $translate = new translate();

      $translate->set_languages($_REQUEST['ffrom'].'|'.$_REQUEST['fto']);
      $translate->set_input_charset(substr($user['language'], 6));
      $translate->set_output_charset(substr($user['language'], 6));

      $nfo=$this->get_key_nfo($_REQUEST['fileid'], $_REQUEST['fkey']);

      $text=stripslashes($nfo[0]['translation']);
      $result=str_replace('<br>', "\n", html_entity_decode($translate->do_translation($text), ENT_QUOTES));

      return($result);
    }
  
    /*
      make a formatted list.<option></option> of existing languages not translated
    */
    function ajax_langs_list()
    {
      $langs=array_flip($this->get_translated_languages());
      $filelangs=$this->get_file_info_by_id($_REQUEST['fileid']);

      $returned="";
      foreach($filelangs['langs'] as $lang)
      {
        if(isset($langs[$lang['language']]))
        {
          unset($langs[$lang['language']]);
        }
      }
      foreach($langs as $key => $lang)
      {
        $returned.="<option value='".$key."'>".$key."</option>";
      } 

      if($returned=='')
      {
        $returned="<div class='throw' style='padding:2px;font-size:90%;'>".
            sprintf(l10n('g001_add_new_language_ko'),
            $this->page_link."&f_tabsheet=langs&action=add")."</div>";
      }
      else
      {
        $returned="<select name='flang' onchange='f=document.forms[\"icreatenlang\"];f.submit();'><option>".l10n('g001_choose_lang')."</option>".$returned."</select>
                    <input type='hidden' name='fileid' value='".$_REQUEST['fileid']."'>
                    <input type='hidden' value='translateto' name='action'>
                    <input type='hidden' value='editing' name='f_tabsheet'>";
      }
      return($returned);
    }

    /*
      return a formatted list of languages 
    */
    function ajax_get_languages_list()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_langs_detail1.tpl');

      $langs=$this->get_langs_list();

      foreach($langs as $lang)
      {
        $local_tpl->assign_block_vars(
          'langs_row',
          array(
            "LANG" => $lang['name'],
            "COUNTRY" => $lang['country'],
            "CHARSET" => strtolower($lang['charset']),
            "CODE" => $lang['code'],
            "NBFILE" => $lang['nbfile'],
            "URL" => $this->page_link."&f_tabsheet=langs&action=edit&code=".urlencode($lang['code'])
          )
        );
      }

      return($local_tpl->parse('body_page', true));
    }

    /*
      export phase 0 :
        create js&html code to prepare export interface
    */
    function ajax_export_phase0()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_maintenance_export_phase0.tpl');


      $sql="SELECT id, nb_elem FROM ".$this->tables['files'];
      $idlist=array();
      if($_POST['f_file_selection_type']=='selected')
      {
        foreach($_POST as $key => $val)
        {
          if((substr($key, 0, 4)=='fid_')&&(is_numeric($val)))
          {
            array_push($idlist, $val);
          }
        }
        $sql.=" WHERE id IN (".implode(',', $idlist).") ";
      }
      $sql.=" ORDER BY nb_elem";

      $result=pwg_query($sql);
      if($result)
      {
        $num=0;
        $list=array('nb' => 0, 'ids' => array());
        while($row=mysql_fetch_assoc($result))
        {
          if($list['nb']+$row['nb_elem']<=$this->my_config['export_maximum_items'])
          {
            $list['nb']+=$row['nb_elem'];
            array_push($list['ids'], $row['id']);
          }
          else
          {
            $local_tpl->assign_block_vars(
              'id_list_row',
              array(
                'NUM' => $num,
                'VALUES' => 'list='.urlencode(implode(',', $list['ids']))
              )
            );
            $num++;
            $list['nb']=$row['nb_elem'];
            $list['ids']=array($row['id']);
          }
        }
        $local_tpl->assign_block_vars(
          'id_list_row',
          array(
            'NUM' => $num,
            'VALUES' => 'list='.urlencode(implode(',', $list['ids']))
          )
        );

      }
      $local_tpl->assign_vars(
        array(
          'OPTION_LIST' => '&f_exported_items='.$_POST['f_exported_items'].'&f_translator_export='.$_POST['f_translator_export'],
          'PROGRESS_BAR_MAX' => $this->my_config['progress_bar_width'],
          'PROGRESS_BAR_COLOR' => $this->my_config['mouse_over_color']
        )
      );

      return($local_tpl->parse('body_page', true));
    }

    /*
      export phase 1
      make .lang.php files
    */
    function ajax_export_phase1()
    {
      if(isset($_POST['list']))
      {
        $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
        $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_maintenance_export_nfo.tpl');

        $list=explode(',', $_POST['list']);
        foreach($list as $fileid)
        {
          $options=array(
            'exported_items' => $_POST['f_exported_items'],
            'translator_export' => $_POST['f_translator_export']
          );
          $result=$this->export($fileid, $options);
          
          if(count($result['errors'])==0)
          {
            $local_tpl->assign_block_vars(
              'result_row',
              array
              (
                'NBEXPORTED' => $result['nfo']['nbexported'],
                'NBELEM' => $result['nfo']['nb_elem'],
                'TYPE' => $result['nfo']['type'],
                'NAME' => $result['nfo']['name'],
                'FILE' => $result['nfo']['file'],
                'FILENAME' => $result['nfo']['exported_file'].' #id:'.$result['nfo']['id'],
                'RESULT_COLOR' => $this->my_config['color_bars3'],
                'EXPORT_RESULT' => l10n('g001_export_ok')
              )
            );
          }
          else
          {
            $local_tpl->assign_block_vars(
              'result_row',
              array
              (
                'NBEXPORTED' => 0,
                'NBELEM' => $result['nfo']['nb_elem'],
                'TYPE' => $result['nfo']['type'],
                'NAME' => $result['nfo']['name'],
                'FILE' => $result['nfo']['file'],
                'FILENAME' => $result['nfo']['exported_file'],
                'RESULT_COLOR' => $this->my_config['color_bars1'],
                'EXPORT_RESULT' => l10n('g001_export_ko').' : '.implode(', ', $result['errors'])
              )
            );
          }
        }
        return($local_tpl->parse('body_page', true));
      }
    }

    /*
      return info about a key
    */
    function ajax_get_key_nfo()
    {
      $nfo=$this->get_key_nfo_by_unique_index($_REQUEST['uindex']);

      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                    dirname($this->filelocation).'/admin/plugin_admin_keynfo.tpl');

        switch($nfo['status'])
        {
          case 'ok':
            $color="color_bars3";
            break;
          case 'valide_it':
            $color="color_bars2";
            break;
          case 'not_translated':
            $color="color_bars1";
            break;
          case 'deleted':
            $color="color_bars4";
            break;
        }


      $local_tpl->assign_vars(
        array(
          'KEY' => $nfo['id'],
          'STATUS' => $nfo['status'],
          'COLOR' => $this->my_config[$color],
          'VERSION' => $nfo['version'],
          'DATE' => ($nfo['status']=='ok')?$nfo['validate_date']:$nfo['date'],
          'TYPE' => $nfo['type'],
          'NAME' => $nfo['name'],
          'FILENAME' => $nfo['file'],
          'FILEDATE' => $nfo['file_date'],
          'LANGUAGE' => $nfo['language']
        )
      );

      if($nfo['status']=='ok')
      {
        $local_tpl->assign_block_vars(
          'user_info',
          array(
            'USERNAME' => $nfo['validate_by_name'],
            'DATE' => $nfo['validate_date']
          )
        );
      }

      return($local_tpl->parse('body_page', true));
    }

    /*
      returns a list of keys who are present in some language file and not in other 
    */
    function ajax_translation_compare_list()
    {
      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");

      $sql="SELECT id, language FROM ".$this->tables['files']."
            WHERE name = '".$_REQUEST['fname']."'
              AND type = '".$_REQUEST['ftype']."'
              AND file = '".$_REQUEST['ffile']."'
            ORDER BY language";
      $result=pwg_query($sql);
      if($result)
      {
        $list = array();
        while($row=mysql_fetch_row($result))
        {
          array_push($list, $row);
        }

        $sql_list=array();
        for($i=0;$i<count($list);$i++)
        {
          $sql_list[$i] = "SELECT DISTINCT id, ".$this->make_compare_columns($i, count($list))."
                            FROM ".$this->tables['keys']."
                            WHERE file_id = ".$list[$i][0]."
                              AND id NOT IN (SELECT DISTINCT id
                                              FROM ".$this->tables['keys']."
                                              WHERE file_id NOT IN (".$this->make_compare_listfile($i, $list).")
                                            ) ";
        }
        $sql=implode(" UNION ", $sql_list)." ORDER BY id";

        $result=pwg_query($sql);
        if($result)
        {
          if(mysql_num_rows($result)>0)
          {
            $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_edit_compare_list.tpl');

            //table header
            for($i=0;$i<count($list);$i++)
            {
              $local_tpl->assign_block_vars(
                'lang_header_cols',
                array(
                  'WIDTH' => $this->my_config['bars_width'],
                  'LABEL' => $list[$i][1]
                )
              );

              $nfo=$this->get_file_info_by_id($list[$i][0]);
              $local_tpl->assign_block_vars(
                'nbelem_header_cols',
                array(
                  'LABEL' => $nfo['nb_elem']
                )
              );

            }

            //keys ; paging not implemted, assuming that number of keys to be updated may not be large
            while($row=mysql_fetch_row($result))
            {
              $datactrl = "";
              $tds = array();
              for($i=1;$i<count($row);$i++)
              {
                if($row[$i]!="")
                {
                  switch($row[$i])
                  {
                    case 'ok':
                      $color="g001_graphbar3";
                      break;
                    case 'valide_it':
                      $color="g001_graphbar2";
                      break;
                    case 'not_translated':
                      $color="g001_graphbar1";
                      break;
                    case 'deleted':
                      $color="g001_graphbar4";
                      break;
                  }
                  array_push($tds, "<div class='".$color."' style='width:".$this->my_config['bars_width']."px'>&nbsp;</div>");
                }
                else
                {
                  array_push($tds, l10n('g001_keys_dont_exists'));
                  $datactrl.= ($datactrl=="")?$list[$i-1][0]:",".$list[$i-1][0];
                }
              }
              $local_tpl->assign_block_vars(
                'keys',
                array(
                  'NAME' => $row[0],
                  'ID' => str2url($row[0]),
                  'DATA' => $datactrl
                )
              );

              for($i=0;$i<count($tds);$i++)
              {
                $local_tpl->assign_block_vars(
                  'keys.nfo_lang_status',
                  array(
                    'TEXT' => $tds[$i]
                  )
                );
              }
            }
          }
          else
          {
            //no differences
            $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_edit_compare_nolist.tpl');
          }
        }
        else
        {
          //sql error !?
        }
      }
      else
      {
        //sql error !?
      }
      return($local_tpl->parse('body_page', true));
    } //ajax_translation_compare_list


    /*
      update keys ; add selected keys to languages
    */
    function ajax_translation_compare_update()
    {
      global $user;

      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_edit_compare_update.tpl');
      $errors=0;
      $text="";
      foreach($_POST as $key => $val)
      {
        $data=explode("#gsep#", $val);
        $text.="key:".$data[1]."files:".$data[0]."<br/>";

        $files=explode(",", $data[0]);
        foreach($files as $key2 => $val2)
        {
          if(!$this->insert_key_translation($val2,$data[1],$user['user_id'],$data[1], 'not_translated'))
          {
            $nfo=$this->get_file_info_by_id($val2);
            $errors++;
            $local_tpl->assign_block_vars(
              'errors',
              array(
                'NFO' => l10n('g001_key')." : ".$data[1].", &nbsp;".l10n('g001_file')." : ".$nfo['language']
              )
            );
          }
        }
      }
      if($errors==0)
      {
        $local_tpl->assign_vars(
          array(
            'TEXT'=>l10n('g001_compare_update_isok'),
            'TEXTCLASS'=>'infos'
          )
        );
      }
      else
      {
        $local_tpl->assign_vars(
          array(
            'TEXT'=>l10n('g001_compare_update_isko'),
            'TEXTCLASS'=>'errors'
          )
        );
      }
      return($local_tpl->parse('body_page', true));
    }




    /*
      erase "deleted keys" and older key's translation version
    */
    function ajax_erase()
    {
      global $user;

      //$local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
/*
      $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_edit_compare_update.tpl');
*/
      $textok="";
      $textko="";
      if($_REQUEST['deleted']=='y')
      {
        $sql="UPDATE ".$this->tables['files']." 
              SET file_date = '0000-00-00 00:00:00' 
              WHERE id IN
                (SELECT DISTINCT file_id
                    FROM ".$this->tables['keys']." 
                    WHERE status = 'deleted')";
        pwg_query($sql);

        $sql="DELETE FROM ".$this->tables['keys']."
              WHERE status = 'deleted';";
        $result=pwg_query($sql);
        if($result)
        {
          $nb=mysql_affected_rows();
          $textok=sprintf(l10n('g001_erase_nb_deleted_deleted'), $nb)."<br/>";
        }
        else
        {
          $textko=l10n('g001_error_erase_deleted')."<br/>";
        }
      }

      if($_REQUEST['older']=='y')
      {
        $sql="DROP TABLE IF EXISTS tmp_translator";
        pwg_query($sql);

        $sql="CREATE TABLE tmp_translator
              SELECT uniqueindex FROM ".$this->tables['keys'].",
              (SELECT id, count(version)as numversion, file_id
                  FROM phpwebgallery_translator_keys
                  GROUP BY id, file_id) as tmp
              WHERE tmp.id = ".$this->tables['keys'].".id
                AND tmp.file_id = ".$this->tables['keys'].".file_id
                AND ".$this->tables['keys'].".version < tmp.numversion
                AND tmp.numversion > 1";
        $result=pwg_query($sql);
        if($result)
        {
          $sql="DELETE FROM ".$this->tables['keys']."
                WHERE uniqueindex IN (SELECT uniqueindex FROM tmp_translator)";
          $result=pwg_query($sql);
          if($result)
          {
            $nb=mysql_affected_rows();
            $textok.=sprintf(l10n('g001_erase_nb_deleted_older'), $nb)."<br/>";
          }
          else
          {
            $textko.=l10n('g001_error_erase_older')."<br/>";
          }
          $sql="DROP TABLE IF EXISTS tmp_translator";
          pwg_query($sql);
        }
      }

      $text="";
      if($textok!="")
      {
        $text.="<div class='infos'>$textok</p>";
        $this->update_file_counter();
      }
      if($textko!="")
      {
        $text.="<div class='errors'>$textko</p>";
      }

      return($text);
    }

    /*
      this function switch translator mode in config file 
    */
    function ajax_switch_translator_mode()
    {
      global $user;

      $done='ko';
      $this->load_config();
      $search=array_search($user['id'], $this->my_config['translator_mode']);
      if($_REQUEST['f_translator_mode']=='on')
      {
        if($search===false)
        {
          //array_push($this->my_config['translator_mode'], $user['user_id']);
          //same result, but seems to be faster
          $this->my_config['translator_mode'][]=$user['id'];
          $this->save_config();
          $done='ok';
        }
      }
      else
      {
        if($search!==false)
        {
          array_splice($this->my_config['translator_mode'], $search, 1);
          $this->save_config();
          $done='ok';
        }
      }

      return("asked:".$_REQUEST['f_translator_mode']."; done:".$done." search result:".$search." user_id:".$user['id']." array:".print_r($this->my_config['translator_mode'],true));
    }


    /*
      analyze differences between server&database file  
    */
    function ajax_analyze_detail()
    {
      global $user;

      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance_analyze_detail.tpl');

      $nbdiff=array(0,0,0);

      if($_REQUEST['fnumfinding']=='1')
      {
        //file on server has changed

        $keydiff=$this->compare_file_and_database_keys($_REQUEST['ftype'],$_REQUEST['fname'],$_REQUEST['ffile'],$_REQUEST['flang']);

        foreach($keydiff['keynfo'] as $key => $val)
        {
          $local_tpl->assign_block_vars(
            'files_row',
            array(
              'KEY' => $key,
              'FINDING' => l10n('g001_finding_num'.$val[0])
            )
          );
          if($val[0]=='4')
          {
            $nbdiff[1]++;
          }
          else
          {
            $nbdiff[2]++;
          }
        }
        $nbdiff[0]=count($keydiff['keynfo']);

        $local_tpl->assign_vars(
          array(
            'TXTDIFF' => sprintf(l10n('g001_analyze_nbdiff'), $nbdiff[0])
          )
        );

        if($nbdiff[1]>0)
        {
          $local_tpl->assign_block_vars(
            'actions1a',
            array(
              'CBIDA' => md5($_REQUEST['ftype'].$_REQUEST['fname'].$_REQUEST['ffile'].$_REQUEST['flang'].'1A'),
              'TYPE' => $_REQUEST['ftype'],
              'NAME' => $_REQUEST['fname'],
              'FILE' => $_REQUEST['ffile'],
              'LANG' => $_REQUEST['flang']
            )
          );
        }

        if($nbdiff[2]>0)
        {
          $local_tpl->assign_block_vars(
            'actions1b',
            array(
              'CBIDB' => md5($_REQUEST['ftype'].$_REQUEST['fname'].$_REQUEST['ffile'].$_REQUEST['flang'].'1B'),
              'TYPE' => $_REQUEST['ftype'],
              'NAME' => $_REQUEST['fname'],
              'FILE' => $_REQUEST['ffile'],
              'LANG' => $_REQUEST['flang']
            )
          );
        }
      }
      elseif($_REQUEST['fnumfinding']=='2')
      {
        //file on server doesn't exists in database
        $langs=array_flip($this->get_translated_languages(true));

        $txtdiff="";
        if(!isset($langs[strtolower($_REQUEST['flang'])]))
        {
          $url="<a href='".$this->page_link."&f_tabsheet=langs&action=add&langcode=".urlencode($_REQUEST['flang'])."'>".sprintf(l10n('g001_finding_num6b'), $_REQUEST['flang'])."</a>";
          $txtdiff=sprintf(l10n('g001_finding_num6'), $url);
        }

        if($txtdiff=="")
        {
          $local_tpl->assign_block_vars(
            'actions2',
            array(
              'CBIDA' => md5($_REQUEST['ftype'].$_REQUEST['fname'].$_REQUEST['ffile'].$_REQUEST['flang'].'2A'),
              'TYPE' => $_REQUEST['ftype'],
              'NAME' => $_REQUEST['fname'],
              'FILE' => $_REQUEST['ffile'],
              'LANG' => $_REQUEST['flang']
            )
          );
        }
        $local_tpl->assign_vars(
          array(
            'VISIBILITY' => 'visibility: hidden;height:0px;',
            'TXTDIFF' => $txtdiff
          )
        );
      }
      elseif($_REQUEST['fnumfinding']=='3')
      {
        //file in database doesn't exists on server
        $local_tpl->assign_block_vars(
          'actions3',
          array(
            'CBIDA' => md5($_REQUEST['ftype'].$_REQUEST['fname'].$_REQUEST['ffile'].$_REQUEST['flang'].'3A'),
            'TYPE' => $_REQUEST['ftype'],
            'NAME' => $_REQUEST['fname'],
            'FILE' => $_REQUEST['ffile'],
            'LANG' => $_REQUEST['flang']
          )
        );

        $local_tpl->assign_vars(
          array(
            'VISIBILITY' => 'visibility: hidden;height:0px;'
          )
        );
      }
      else
      {
        //??
      }

      return($local_tpl->parse('body_page', true));
    }


    /*
      return result of analyze
    */
    function ajax_analyze_list()
    {
      global $user;

      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance_analyze_list.tpl');

      //make a scan of directories
      $disk_filesdatas=array();
      $files=$this->search_languages_files(PHPWG_ROOT_PATH."/language", $this->my_config["charset"]);
      $files=array_merge($files, $this->search_languages_files(PHPWG_ROOT_PATH."/plugins", $this->my_config["charset"]));
      foreach($files as $file)
      {
        $nfo=$this->analyze_filename($file);
        $disk_filesdatas[$nfo['filename']]=$nfo;
      }

      $base_datasfiles=$this->get_all_files();

      $numfile=0;
      foreach($base_datasfiles as $file)
      {
        if(isset($disk_filesdatas[$file['filename']]))
        {
          if(date('Y-m-d H:i:s', $disk_filesdatas[$file['filename']]['modified'])!=$file['file_date'])
          {
            // file on server has changed
            $numfile++;
            $local_tpl->assign_block_vars(
              'diff_files',
              array(
                'ID' => $numfile,
                'TYPE' => $file['type'],
                'NAME' => $file['name'],
                'FILE' => $file['file'],
                'LANG' => $file['language'],
                'NUMFINDING' => '1',
                'FINDING' => l10n('g001_finding_num1').' ('.$file['filename'].')'
              )
            );
          }
          unset($disk_filesdatas[$file['filename']]);
        }
        else
        {
          // file on server does not exist
          $numfile++;
          $local_tpl->assign_block_vars(
            'diff_files',
            array(
              'ID' => $numfile,
              'TYPE' => $file['type'],
              'NAME' => $file['name'],
              'FILE' => $file['file'],
              'LANG' => $file['language'],
              'NUMFINDING' => '3',
              'FINDING' => l10n('g001_finding_num3').' ('.$file['filename'].')'
            )
          );
        }
      }

      //files in array are file existing on serveur and doesn't exist on database
      foreach($disk_filesdatas as $file)
      {
        $numfile++;
        $local_tpl->assign_block_vars(
          'diff_files',
          array(
            'ID' => $numfile,
            'TYPE' => $file['type'],
            'NAME' => $file['name'],
            'FILE' => $file['file'],
            'LANG' => $file['lang'],
            'NUMFINDING' => '2',
            'FINDING' => l10n('g001_finding_num2').' ('.$file['filename'].')'
          )
        );
      }

      if($numfile>0)
      {
        return($local_tpl->parse('body_page', true));
      }
      else
      {
        return("<p>".l10n('g001_analyze_nodiff')."</p>");
      }
    }


    /*
      return result of analyze
    */
    function ajax_analyze_upgrade()
    {
      global $user;

      $local_tpl = new Template(TRANSLATOR_PATH."admin/", "");
      $local_tpl->set_filename('body_page',
                              dirname($this->filelocation).'/admin/plugin_admin_maintenance_analyze_upgrade.tpl');

      foreach($_POST as $key => $val)
      {
        $nfo=array();
        $values=explode(';', $val);
        //0:type, 1:name, 2:file, 3:lang, 4:action

        $result=true;
        if($values[4]=='1A')
        {
          //import from file to database the new keys
          $keydiff=$this->compare_file_and_database_keys($values[0],$values[1],$values[2],$values[3]);
          foreach($keydiff['keynfo'] as $key2 => $val2)
          {
            if($val2[0]=='4')
            {
              $result2=$this->insert_key_translation($keydiff['filenfo']['id'], $key2, $user['id'], $val2[1], 'valide_it');
              if(!$result2) { $result=false; }
            }
          }

          $local_tpl->assign_block_vars(
            'result_row',
            array(
              'RESULT_COLOR' => ($result==true)?$this->my_config['color_bars3']:$this->my_config['color_bars1'],
              'TYPE' => $values[0],
              'NAME' => $values[1],
              'FILE' => $values[2],
              'LANG' => $values[3],
              'ACTION' => l10n('g001_do_upgrade_1a'),
              'ACTION_RESULT' => ($result==true)?l10n('g001_ok'):l10n('g001_ko')
            )
          );
          $this->update_date_file($keydiff['filenfo']);
        }
        elseif($values[4]=='1B')
        {
          //delete in database keys who aren't in file
          $keydiff=$this->compare_file_and_database_keys($values[0],$values[1],$values[2],$values[3]);
          foreach($keydiff['keynfo'] as $key2 => $val2)
          {
            if($val2[0]=='5')
            {
              $result2=$this->delete_key($keydiff['filenfo']['id'], $key2, $user['id']);
              if(!$result2) { $result=false; }
            }
          }

          $local_tpl->assign_block_vars(
            'result_row',
            array(
              'RESULT_COLOR' => ($result==true)?$this->my_config['color_bars3']:$this->my_config['color_bars1'],
              'TYPE' => $values[0],
              'NAME' => $values[1],
              'FILE' => $values[2],
              'LANG' => $values[3],
              'ACTION' => l10n('g001_do_upgrade_1b'),
              'ACTION_RESULT' => ($result==true)?l10n('g001_ok'):l10n('g001_ko')
            )
          );
          $this->update_date_file($keydiff['filenfo']);
        }
        elseif($values[4]=='2A')
        {
          //import complete file into database
          
          if($values[0]=='pwg')
          {
            // ./language/{lang}/{file}
            $fic=PHPWG_ROOT_PATH.'/language/'.$values[3].'/'.$values[2];
          }
          else
          {
            // ./plugins/{name}/language/{lang}/{file}
            $fic=PHPWG_PLUGINS_PATH.$values[1].'/language/'.$values[3].'/'.$values[2];
          }

          $nfo[0]=$this->analyze_filename($fic);

          $result=$this->load_lang_files('','',$nfo,$user['id']);

          $local_tpl->assign_block_vars(
            'result_row',
            array(
              'RESULT_COLOR' => (count($result)==0)?$this->my_config['color_bars3']:$this->my_config['color_bars1'],
              'TYPE' => $values[0],
              'NAME' => $values[1],
              'FILE' => $values[2],
              'LANG' => $values[3],
              'ACTION' => l10n('g001_do_upgrade_2a'),
              'ACTION_RESULT' => (count($result)==0)?l10n('g001_ok'):l10n('g001_ko')
            )
          );
        }
        elseif($values[4]=='3A')
        {
          //purge file from database
          $nfo=$this->get_file_info($values[0],$values[1],$values[2],$values[3]);
          $result=$this->erase_file_from_database($nfo[$values[3]]['id']);

          $local_tpl->assign_block_vars(
            'result_row',
            array(
              'RESULT_COLOR' => ($result==true)?$this->my_config['color_bars3']:$this->my_config['color_bars1'],
              'TYPE' => $values[0],
              'NAME' => $values[1],
              'FILE' => $values[2],
              'LANG' => $values[3],
              'ACTION' => l10n('g001_do_upgrade_3a'),
              'ACTION_RESULT' => ($result==true)?l10n('g001_ok'):l10n('g001_ko')
            )
          );
        }
        else
        {
          // ???
        }
      }


      return($local_tpl->parse('body_page', true));
    }


  } //class

?>
